local _, iTF = ...

--Defaults iTFConfig.frame.spacing = 4
-- iTFConfig.frame.height = 50
-- iTFConfig.frame.castBarHeight = 25
-- iTFConfig.statusbar.castBarHeight
function iTF:LoadDefaults(force)
	local defaults = {
		['anchor'] = {
			['x'] = 20,
			['y'] = 600,
		},
		['grow'] = 'TOPLEFT',
		['frame'] = {
			['width'] = 60,
			['height'] = 30,
			['spacing'] = 2,
			['castBarHeight'] = 8,
		},
		['icon'] = {
			['width'] = 20,
			['height'] = 20,
			['pos'] = 'BOTTOMLEFT',
			['max'] = 4,
			['grow'] = 'LEFT',
			['x'] = 1,
			['y'] = 1,
		},
		['raidIcon'] = {
			['size'] = 20,
			['pos'] = 'CENTER',
			['x'] = 0,
			['y'] = 0,
		},
		['text'] = {
			['pos'] = 'TOPLEFT',
			['size'] = 11,
			['font'] = 'Fonts\\ARIALN.TTF',
			['x'] = 2,
			['y'] = -2,
		},
		['statusbar'] = {
			['texture'] = 'Interface\\Buttons\\WHITE8x8',
		},
		['border'] = {
			['currentTarget'] = {
				['enable'] = true,
				['color'] = {1,1,1,1},
			},
		},
		['colS'] = 5,
		['colors'] = {
			['backdrop'] = {
				['bg'] = {0.1,0.1,0.1,0.9},
				['border'] = {0,0,0,1},
			},
			['statusbar'] = {
				['main'] = {0.5,0,0,0.8},
				['cast'] = {0.5,0.5,0.5,0.8},
			},
			['text'] = {
				['main'] = {1,1,1,1},
				['cast'] = {1,1,0,1},
				['immune'] = {1,0,0,1},
			},
			['targeting'] = {
				['main'] = {1,1,1,1},
			},
		},
		['glows'] = {
			['outOfCombat'] = {
				['pos'] = 'glowLeft',
				['color'] = {0.5,0.5,0.5,0.8},
				['enable'] = true,
			},
		},
	}
	if force then
		iTFConfig = defaults
	else
		for k,v in pairs(defaults) do
			if iTFConfig[k] == nil then
				iTFConfig[k] = v
			elseif type(v) == 'table' then
				for deeperKey, deeperValue in pairs(v) do
					if iTFConfig[k][deeperKey] == nil then
						iTFConfig[k][deeperKey] = deeperValue
					elseif type(deeperValue) == 'table' then
						for inceptionLevelKey, inceptionLevelValue in pairs(deeperValue) do
							if iTFConfig[k][deeperKey][inceptionLevelKey] == nil then
								iTFConfig[k][deeperKey][inceptionLevelKey] = inceptionLevelValue
							elseif type(inceptionLevelValue) == 'table' then
								for wtfKey, wtfValue in pairs(inceptionLevelValue) do
									if iTFConfig[k][deeperKey][inceptionLevelKey][wtfKey] == nil then
										iTFConfig[k][deeperKey][inceptionLevelKey][wtfKey] = wtfValue
									end
								end
							end
						end
					end
				end
			end
		end
	end
end
local function loadProfile(profile)
	if profile == 'default' then
		iTF:LoadDefaults(true)
	end
end

local function MainFrameBG(show)
	if show then
		iTF.mainFrame.tex:Show()
		iTF.show = true
	else
		iTF.mainFrame.tex:Hide()
		iTF.show = nil
	end
end

local options = {
	type = "group",
	args = {
		ShowBackground = {
			name = 'Show background',
			desc = 'Show max size',
			type = 'toggle',
			set = function(info,val) MainFrameBG(val) end,
			get = function(info) return iTF.show end
		},
		--[[
		toggleLock = {
			name = 'Toggle Lock',
			desc = 'lock frame',
			type = 'toggle',
			set = function(info,val) MainFrameBG(val) end,
			get = function(info) return iTF.show end
		},
		--]]
		frameOptions={
			name = 'Frame',
			type = 'group',
			order = 1,
			args = {
				xPos = {
					name = 'Horizontal position',
					desc = 'Horizontal position',
					type = 'range',
					order = 1,
					min = 0,
					max = math.floor(GetScreenWidth()),
					step = 1,
					
					set = function(info,val)
						iTFConfig.anchor.x = val
						iTF:updatePos(true)
					end,
					get = function(info) return iTFConfig.anchor.x end
				},
				yPos = {
					name = 'Vertical position',
					desc = 'Vertical position',
					type = 'range',
					order = 2,
					min = 0,
					max = math.floor(GetScreenHeight()),
					step = 1,
					set = function(info,val)
						iTFConfig.anchor.y = val
						iTF:updatePos()
					end,
					get = function(info) return iTFConfig.anchor.y end
				},
				rows = {
					name = 'Rows',
					desc = 'Row size',
					type = 'range',
					order = 3,
					min = 1,
					max = 30,
					step = 1,
					set = function(info,val)
						iTFConfig.colS = val
						iTF:updatePos()
					end,
					get = function(info) return iTFConfig.colS end
				},
				grow = {
					name = 'Grow',
					desc = 'Grow to',
					type = 'select',
					order = 4,
					values = {['TOPRIGHT'] = 'Down & Left', ['TOPLEFT'] = 'Down & Right', ['BOTTOMRIGHT'] = 'Up & Left', ['BOTTOMLEFT'] = 'Up & Right'},
					set = function(info,val)
						iTFConfig.grow = val
						iTF:updatePos()
					end,
					get = function(info) return iTFConfig.grow end
				},
				spacing = {
					name = 'Spacing',
					desc = 'frameSpacing',
					type = 'range',
					order = 5,
					min = 2,
					max = 100,
					step = 1,
					set = function(info,val)
						iTFConfig.frame.spacing = val
						iTF:updatePos()
					end,
					get = function(info) return iTFConfig.frame.spacing end
				},
				size = {
					name = 'Size',
					type = 'group',
					order = 1,
					args = {
						width = {
							name = 'Width',
							desc = 'Unit frame width',
							type = 'range',
							order = 1,
							min = 10,
							max = 200,
							step = 1,
							
							set = function(info,val)
								iTFConfig.frame.width = val
								iTF:updateSize()
							end,
							get = function(info) return iTFConfig.frame.width end
						},
						height = {
							name = 'Height',
							desc = 'Unit frame height',
							type = 'range',
							order = 2,
							min = 10,
							max = 200,
							step = 1,
							set = function(info,val)
								iTFConfig.frame.height = val
								iTF:updateSize()
							end,
							get = function(info) return iTFConfig.frame.height end
						},
					},
				},
				text = {
					name = 'Text',
					type = 'group',
					order = 2,
					args = {
						size = {
							name = 'Size',
							desc = 'Font size',
							type = 'range',
							order = 1,
							min = 5,
							max = 50,
							step = 1,
							set = function(info,val)
								iTFConfig.text.size = val
								iTF:updateFrames('text')
							end,
							get = function(info) return iTFConfig.text.size end
						},
						textPosition = {
							name = 'Text Position',
							desc = 'Text Position',
							type = 'select',
							order = 2,
							values = {['TOPLEFT'] = 'Top left', ['TOPRIGHT'] = 'Top right', ['TOP'] = 'Top', ['LEFT'] = 'Left', ['CENTER'] = 'Center', ['RIGHT'] = 'Right', ['BOTTOMRIGHT'] = 'Bottom right', ['BOTTOMLEFT'] = 'Bottom left', ['BOTTOM'] = 'Bottom'},
							set = function(info,val)
								iTFConfig.text.pos = val
								iTF:updateFrames('text')
							end,
							get = function(info) return iTFConfig.text.pos end
						},
						horizontal = {
							name = 'Horizontal position',
							desc = 'Horizontal position',
							type = 'range',
							order = 3,
							min = -50,
							max = 50,
							step = 1,
							set = function(info,val)
								iTFConfig.text.x = val
								iTF:updateFrames('text')
							end,
							get = function(info) return iTFConfig.text.x end
						},
						vertical = {
							name = 'Vertical position',
							desc = 'Vertical position',
							type = 'range',
							order = 4,
							min = -50,
							max = 50,
							step = 1,
							set = function(info,val)
								iTFConfig.text.y = val
								iTF:updateFrames('text')
							end,
							get = function(info) return iTFConfig.text.y end
						},
						nameColor = {
							name = 'Name color',
							desc = 'Name text color',
							type = 'color',
							order = 5,
							hasAlpha = true,
							set = function(info,r,g,b,a)
								iTFConfig.colors.text.main = {r,g,b,a}
								iTF:updateFrames('text')
							end,
							get = function(info)
								return unpack(iTFConfig.colors.text.main)
							end,
						},
						castColor = {
							name = 'Cast color',
							desc = 'Cast text color',
							type = 'color',
							order = 6,
							hasAlpha = true,
							set = function(info,r,g,b,a)
								iTFConfig.colors.text.cast = {r,g,b,a}
								iTF:updateFrames('text')
							end,
							get = function(info)
								return unpack(iTFConfig.colors.text.cast)
							end,
						},
						immuneColor = {
							name = 'Non interruptible color',
							desc = 'Non interruptible cast text color',
							type = 'color',
							order = 7,
							hasAlpha = true,
							set = function(info,r,g,b,a)
								iTFConfig.colors.text.immune = {r,g,b,a}
								iTF:updateFrames('text')
							end,
							get = function(info)
								return unpack(iTFConfig.colors.text.immune)
							end,
						},
					},
				},
				castBar = {
					name = 'Cast bar',
					type = 'group',
					order = 3,
					args = {
						height = {
							name = 'Height',
							desc = 'Cast bar height',
							type = 'range',
							order = 1,
							min = 1,
							max = 60,
							step = 1,
							set = function(info,val)
								iTFConfig.frame.castBarHeight = val
								iTF:updateSize()
							end,
							get = function(info) return iTFConfig.frame.castBarHeight end
						},
						color = {
							name = 'Cast bar color',
							desc = 'Cast bar color',
							type = 'color',
							order = 2,
							hasAlpha = true,
							set = function(info,r,g,b,a)
								iTFConfig.colors.statusbar.cast = {r,g,b,a}
								iTF:updateFrames('castBarColor')
							end,
							get = function(info)
								return unpack(iTFConfig.colors.statusbar.cast)
							end,
						},
					},
				},
				statusbar = {
					name = 'Health bar',
					type = 'group',
					order = 2,
					args = {
						color = {
							name = 'Bar color',
							desc = 'Bar color',
							type = 'color',
							order = 1,
							hasAlpha = true,
							set = function(info,r,g,b,a)
								iTFConfig.colors.statusbar.main = {r,g,b,a}
								iTF:updateFrames('statusBarColor')
							end,
							get = function(info)
								return unpack(iTFConfig.colors.statusbar.main)
							end,
						},
					},
				},
				raidIcon = {
					name = 'Raid marker',
					type = 'group',
					order = 3,
					args = {
						size = {
							name = 'Size',
							desc = 'Icon size',
							type = 'range',
							order = 1,
							min = 5,
							max = 100,
							step = 1,
							set = function(info,val)
								iTFConfig.raidIcon.size = val
								iTF:updateFrames('raidIcon')
							end,
							get = function(info) return iTFConfig.raidIcon.size end
						},
						textPosition = {
							name = 'Icon Position',
							desc = 'Icon Position',
							type = 'select',
							order = 2,
							values = {['TOPLEFT'] = 'Top left', ['TOPRIGHT'] = 'Top right', ['TOP'] = 'Top', ['LEFT'] = 'Left', ['CENTER'] = 'Center', ['RIGHT'] = 'Right', ['BOTTOMRIGHT'] = 'Bottom right', ['BOTTOMLEFT'] = 'Bottom left', ['BOTTOM'] = 'Bottom'},
							set = function(info,val)
								iTFConfig.raidIcon.pos = val
								iTF:updateFrames('raidIcon')
							end,
							get = function(info) return iTFConfig.raidIcon.pos end
						},
						horizontal = {
							name = 'Horizontal position',
							desc = 'Horizontal position',
							type = 'range',
							order = 3,
							min = -50,
							max = 50,
							step = 1,
							set = function(info,val)
								iTFConfig.raidIcon.x = val
								iTF:updateFrames('raidIcon')
							end,
							get = function(info) return iTFConfig.raidIcon.x end
						},
						vertical = {
							name = 'Vertical position',
							desc = 'Vertical position',
							type = 'range',
							order = 4,
							min = -50,
							max = 50,
							step = 1,
							set = function(info,val)
								iTFConfig.raidIcon.y = val
								iTF:updateFrames('raidIcon')
							end,
							get = function(info) return iTFConfig.raidIcon.y end
						},
					},
				},
			},
		},
		auras={
			name = 'Auras',
			type = 'group',
			order = 2,
			args = {
				icon = {
					name = 'Icon',
					type = 'group',
					order = 1,
					args = {
						height = {
							name = 'Height',
							desc = 'Icon Height',
							type = 'range',
							order = 1,
							min = 1,
							max = 60,
							step = 1,
							set = function(info,val)
								iTFConfig.icon.height = val
								iTF:updateFrames('auraIcon')
							end,
							get = function(info) return iTFConfig.icon.height end
						},
						width = {
							name = 'Width',
							desc = 'Icon width',
							type = 'range',
							order = 2,
							min = 1,
							max = 60,
							step = 1,
							set = function(info,val)
								iTFConfig.icon.width = val
								iTF:updateFrames('auraIcon')
							end,
							get = function(info) return iTFConfig.icon.width end
						},
						horizontal = {
							name = 'Horizontal position',
							desc = 'Horizontal position',
							type = 'range',
							order = 3,
							min = -50,
							max = 50,
							step = 1,
							set = function(info,val)
								iTFConfig.icon.x = val
								iTF:updateFrames('auraIcon')
							end,
							get = function(info) return iTFConfig.icon.x end
						},
						vertical = {
							name = 'Vertical position',
							desc = 'Vertical position',
							type = 'range',
							order = 4,
							min = -50,
							max = 50,
							step = 1,
							set = function(info,val)
								iTFConfig.icon.y = val
								iTF:updateFrames('auraIcon')
							end,
							get = function(info) return iTFConfig.icon.y end
						},
						position = {
							name = 'Icon position',
							desc = 'Icon position',
							type = 'select',
							order = 5,
							values = {['TOPLEFT'] = 'Top left', ['TOPRIGHT'] = 'Top right', ['TOP'] = 'Top', ['LEFT'] = 'Left', ['CENTER'] = 'Center', ['RIGHT'] = 'Right', ['BOTTOMRIGHT'] = 'Bottom right', ['BOTTOMLEFT'] = 'Bottom left', ['BOTTOM'] = 'Bottom'},
							set = function(info,val)
								iTFConfig.icon.pos = val
								iTF:updateFrames('auraIcon')
							end,
							get = function(info) return iTFConfig.icon.pos end
						},
						grow = {
							name = 'Grow',
							desc = 'Grow',
							type = 'select',
							order = 6,
							values = {['LEFT'] = 'RIGHT', ['RIGHT'] = 'LEFT', ['TOP'] = 'BOTTOM', ['BOTTOM'] = 'TOP'},
							set = function(info,val)
								iTFConfig.icon.grow = val
								iTF:updateFrames('auraIcon')
							end,
							get = function(info) return iTFConfig.icon.grow end
						},
						maxAuras = {
							name = 'Max auras',
							desc = 'Max auras',
							type = 'range',
							order = 7,
							min = 1,
							max = 8,
							step = 1,
							set = function(info,val)
								iTFConfig.icon.max = val
								iTF:updateFrames('auraIcon')
							end,
							get = function(info) return iTFConfig.icon.max end
						},
					},
				},
			},
		},
		indicators={
			name = 'Indicators',
			type = 'group',
			order = 3,
			args = {
				outOfCombat = {
					name = 'Out of combat',
					type = 'group',
					order = 1,
					args = {
						enable = {
							name = 'Enable',
							desc = 'Enable out of combat glow',
							order = 1,
							type = 'toggle',
							set = function(info,val) iTFConfig.glows.outOfCombat.enable = val end,
							get = function(info) return iTFConfig.glows.outOfCombat.enable end,
						},
						pos = {
							name = 'Position',
							type = 'select',
							order = 2,
							values = {['glowLeft'] = 'Left', ['glowRight'] = 'Right', ['glowTop'] = 'Top', ['glowBottom'] = 'Bottom'},
							set = function(info,val)
								iTFConfig.glows.outOfCombat.pos = val
								iTF:updateFrames('glow')
							end,
							get = function(info) return iTFConfig.glows.outOfCombat.pos end,
						},
						color = {
							name = 'Color',
							type = 'color',
							order = 3,
							hasAlpha = true,
							set = function(info,r,g,b,a)
								iTFConfig.glows.outOfCombat.color = {r,g,b,a}
								iTF:updateFrames('glow')
							end,
							get = function(info)
								return unpack(iTFConfig.glows.outOfCombat.color)
							end,
						},
					},
				},
				targeting = {
					name = 'Target',
					type = 'group',
					order = 1,
					args = {
						enable = {
							name = 'Enable',
							desc = 'Enable current target border',
							order = 1,
							type = 'toggle',
							set = function(info,val) iTFConfig.border.currentTarget.enable = val end,
							get = function(info) return iTFConfig.border.currentTarget.enable end,
						},
						color = {
							name = 'Color',
							type = 'color',
							order = 3,
							hasAlpha = true,
							set = function(info,r,g,b,a)
								iTFConfig.border.currentTarget.color = {r,g,b,a}
								iTF:updateFrames('border')
							end,
							get = function(info)
								return unpack(iTFConfig.border.currentTarget.color)
							end,
						},
					},
				},
			},
		},
		layouts={
			name = 'Layouts',
			type = 'group',
			order = 4,
			args = {
				defaults = {
					order = 1,
					name = 'Reset to Defaults',
					type = 'execute',
					func = function() 
						loadProfile('default')
						iTF:updateFrames()
					end,
				},
			},
		},
	},
}
LibStub('AceConfig-3.0'):RegisterOptionsTable('iTargetingFrames', options)
local AceConfigDialog = LibStub('AceConfigDialog-3.0')
AceConfigDialog:SetDefaultSize('iTargetingFrames', 600,500)

function iTF:toggleConfig(hide)
	AceConfigDialog:Open('iTargetingFrames')
end
